// src/index.ts
import { Command } from "commander";

// package.json
var package_default = {
  name: "aws-cost-cli",
  version: "0.1.2",
  description: "Simple CLI to get your AWS costs",
  type: "module",
  author: {
    name: "Kamran Ahmed",
    email: "kamranahmed.se@gmail.com",
    url: "https://github.com/kamranahmedse"
  },
  files: [
    "!tests/**/*",
    "dist/**/*",
    "!dist/**/*.js.map",
    "bin/**/*"
  ],
  bin: {
    "aws-cost": "./bin/index.js"
  },
  scripts: {
    build: "tsup",
    dev: "tsup --watch",
    prebuild: "run-s clean",
    predev: "run-s clean",
    clean: "rm -rf dist",
    typecheck: "tsc --noEmit",
    test: 'echo "Error: no test specified" && exit 1'
  },
  keywords: [
    "aws",
    "cost",
    "cli",
    "aws-cost",
    "aws-cost-cli",
    "aws-costs",
    "typescript",
    "aws cli"
  ],
  license: "MIT",
  repository: {
    type: "git",
    url: "https://github.com/kamranahmedse/aws-cost-cli.git"
  },
  engines: {
    node: ">=12.0"
  },
  bugs: {
    url: "https://github.com/kamranahmedse/aws-cost-cli/issues"
  },
  homepage: "https://github.com/kamranahmedse/aws-cost-cli#readme",
  dependencies: {
    "aws-sdk": "^2.1299.0",
    chalk: "^5.2.0",
    commander: "^10.0.0",
    dayjs: "^1.11.7",
    dotenv: "^16.0.3",
    ora: "^6.1.2"
  },
  devDependencies: {
    "@types/node": "^18.11.18",
    "npm-run-all": "^4.1.5",
    "ts-node": "^10.9.1",
    tsup: "^6.5.0",
    typescript: "^4.9.4"
  }
};

// src/account.ts
import AWS from "aws-sdk";

// src/spinner.ts
import ora from "ora";
var spinner;
function showSpinner(text) {
  if (!spinner) {
    spinner = ora({ text: "" }).start();
  }
  spinner.text = text;
}
function hideSpinner() {
  if (!spinner) {
    return;
  }
  spinner.stop();
}

// src/account.ts
async function getAccountAlias(awsConfig2) {
  var _a;
  showSpinner("Getting account alias");
  const iam = new AWS.IAM(awsConfig2);
  const accountAliases = await iam.listAccountAliases().promise();
  const foundAlias = (_a = accountAliases == null ? void 0 : accountAliases["AccountAliases"]) == null ? void 0 : _a[0];
  if (foundAlias) {
    return foundAlias;
  }
  const sts = new AWS.STS(awsConfig2);
  const accountInfo = await sts.getCallerIdentity().promise();
  return (accountInfo == null ? void 0 : accountInfo.Account) || "";
}

// src/config.ts
import dotenv from "dotenv";
import fs from "node:fs";
function loadConfig(path = void 0) {
  if (typeof path === "boolean") {
    console.error("Invalid path to the config file");
    process.exit(1);
  }
  if (!path) {
    dotenv.config();
  } else {
    if (!fs.existsSync(path)) {
      console.error(`Config file not found: ${path}`);
      process.exit(1);
    }
    dotenv.config({ path });
  }
  const envConfig = readEnvConfig();
  if (!envConfig.awsAccessKey || !envConfig.awsSecretKey || !envConfig.awsRegion) {
    console.error("Missing AWS credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION");
    process.exit(1);
  }
}
function readEnvConfig() {
  return {
    awsAccessKey: process.env.AWS_ACCESS_KEY,
    awsSecretKey: process.env.AWS_SECRET_KEY,
    awsRegion: process.env.AWS_REGION || "us-east-1"
  };
}
function getAwsCredentials() {
  const envConfig = readEnvConfig();
  if (!envConfig.awsAccessKey || !envConfig.awsSecretKey || !envConfig.awsRegion) {
    throw new Error("Missing AWS credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION");
  }
  return {
    credentials: {
      accessKeyId: envConfig.awsAccessKey,
      secretAccessKey: envConfig.awsSecretKey
    },
    region: envConfig.awsRegion
  };
}

// src/cost.ts
import AWS2 from "aws-sdk";
import dayjs from "dayjs";
async function getRawCostByService(awsConfig2) {
  showSpinner("Getting pricing data");
  const costExplorer = new AWS2.CostExplorer(awsConfig2);
  const endDate = dayjs().subtract(1, "day");
  const startDate = endDate.subtract(65, "day");
  const pricingData = await costExplorer.getCostAndUsage({
    TimePeriod: {
      Start: startDate.format("YYYY-MM-DD"),
      End: endDate.format("YYYY-MM-DD")
    },
    Granularity: "DAILY",
    Filter: {
      Not: {
        Dimensions: {
          Key: "RECORD_TYPE",
          Values: ["Credit", "Refund", "Upfront", "Support"]
        }
      }
    },
    Metrics: ["UnblendedCost"],
    GroupBy: [
      {
        Type: "DIMENSION",
        Key: "SERVICE"
      }
    ]
  }).promise();
  const costByService = {};
  for (const day of pricingData.ResultsByTime) {
    for (const group of day.Groups) {
      const serviceName = group.Keys[0];
      const cost = group.Metrics.UnblendedCost.Amount;
      const costDate = day.TimePeriod.End;
      costByService[serviceName] = costByService[serviceName] || {};
      costByService[serviceName][costDate] = parseFloat(cost);
    }
  }
  return costByService;
}
function calculateServiceTotals(rawCostByService) {
  const totals = {
    lastMonth: 0,
    thisMonth: 0,
    last7Days: 0,
    yesterday: 0
  };
  const totalsByService = {
    lastMonth: {},
    thisMonth: {},
    last7Days: {},
    yesterday: {}
  };
  const startOfLastMonth = dayjs().subtract(1, "month").startOf("month");
  const startOfThisMonth = dayjs().startOf("month");
  const startOfLast7Days = dayjs().subtract(7, "day");
  const startOfYesterday = dayjs().subtract(1, "day");
  for (const service of Object.keys(rawCostByService)) {
    const servicePrices = rawCostByService[service];
    let lastMonthServiceTotal = 0;
    let thisMonthServiceTotal = 0;
    let last7DaysServiceTotal = 0;
    let yesterdayServiceTotal = 0;
    for (const date of Object.keys(servicePrices)) {
      const price = servicePrices[date];
      const dateObj = dayjs(date);
      if (dateObj.isSame(startOfLastMonth, "month")) {
        lastMonthServiceTotal += price;
      }
      if (dateObj.isSame(startOfThisMonth, "month")) {
        thisMonthServiceTotal += price;
      }
      if (dateObj.isSame(startOfLast7Days, "week") && !dateObj.isSame(startOfYesterday, "day")) {
        last7DaysServiceTotal += price;
      }
      if (dateObj.isSame(startOfYesterday, "day")) {
        yesterdayServiceTotal += price;
      }
    }
    totalsByService.lastMonth[service] = lastMonthServiceTotal;
    totalsByService.thisMonth[service] = thisMonthServiceTotal;
    totalsByService.last7Days[service] = last7DaysServiceTotal;
    totalsByService.yesterday[service] = yesterdayServiceTotal;
    totals.lastMonth += lastMonthServiceTotal;
    totals.thisMonth += thisMonthServiceTotal;
    totals.last7Days += last7DaysServiceTotal;
    totals.yesterday += yesterdayServiceTotal;
  }
  return {
    totals,
    totalsByService
  };
}
async function getTotalCosts(awsConfig2) {
  const rawCosts = await getRawCostByService(awsConfig2);
  const totals = calculateServiceTotals(rawCosts);
  return totals;
}

// src/printers/fancy.ts
import chalk from "chalk";
function printFancy(accountAlias, totals, isSummary = false) {
  hideSpinner();
  console.clear();
  const allServices = Object.keys(totals.totalsByService.lastMonth);
  const sortedServiceNames = allServices.sort((a, b) => b.length - a.length);
  const maxServiceLength = sortedServiceNames.reduce((max, service) => {
    return Math.max(max, service.length);
  }, 0) + 1;
  const totalLastMonth = chalk.green(`$${totals.totals.lastMonth.toFixed(2)}`);
  const totalThisMonth = chalk.green(`$${totals.totals.thisMonth.toFixed(2)}`);
  const totalLast7Days = chalk.green(`$${totals.totals.last7Days.toFixed(2)}`);
  const totalYesterday = chalk.bold.yellowBright(`$${totals.totals.yesterday.toFixed(2)}`);
  console.log("");
  console.log(`${"AWS Cost Report:".padStart(maxServiceLength + 1)} ${chalk.bold.yellow(accountAlias)}`);
  console.log("");
  console.log(`${"Last Month".padStart(maxServiceLength)}: ${totalLastMonth}`);
  console.log(`${"This Month".padStart(maxServiceLength)}: ${totalThisMonth}`);
  console.log(`${"Last 7 days".padStart(maxServiceLength)}: ${totalLast7Days}`);
  console.log(`${chalk.bold("Yesterday".padStart(maxServiceLength))}: ${totalYesterday}`);
  console.log("");
  if (isSummary) {
    return;
  }
  const headerPadLength = 11;
  const serviceHeader = chalk.white("Service".padStart(maxServiceLength));
  const lastMonthHeader = chalk.white(`Last Month`.padEnd(headerPadLength));
  const thisMonthHeader = chalk.white(`This Month`.padEnd(headerPadLength));
  const last7DaysHeader = chalk.white(`Last 7 Days`.padEnd(headerPadLength));
  const yesterdayHeader = chalk.bold.white("Yesterday".padEnd(headerPadLength));
  console.log(`${serviceHeader} ${lastMonthHeader} ${thisMonthHeader} ${last7DaysHeader} ${yesterdayHeader}`);
  for (let service of sortedServiceNames) {
    const serviceLabel = chalk.cyan(service.padStart(maxServiceLength));
    const lastMonthTotal = chalk.green(
      `$${totals.totalsByService.lastMonth[service].toFixed(2)}`.padEnd(headerPadLength)
    );
    const thisMonthTotal = chalk.green(
      `$${totals.totalsByService.thisMonth[service].toFixed(2)}`.padEnd(headerPadLength)
    );
    const last7DaysTotal = chalk.green(
      `$${totals.totalsByService.last7Days[service].toFixed(2)}`.padEnd(headerPadLength)
    );
    const yesterdayTotal = chalk.bold.yellowBright(
      `$${totals.totalsByService.yesterday[service].toFixed(2)}`.padEnd(headerPadLength)
    );
    console.log(`${serviceLabel} ${lastMonthTotal} ${thisMonthTotal} ${last7DaysTotal} ${yesterdayTotal}`);
  }
}

// src/printers/json.ts
function printJson(accountAlias, totalCosts, isSummary = false) {
  hideSpinner();
  if (isSummary) {
    console.log(
      JSON.stringify(
        {
          account: accountAlias,
          totals: totalCosts.totals
        },
        null,
        2
      )
    );
    return;
  }
  console.log(
    JSON.stringify(
      {
        account: accountAlias,
        ...totalCosts
      },
      null,
      2
    )
  );
}

// src/printers/text.ts
function printPlainSummary(accountAlias, totals) {
  hideSpinner();
  console.clear();
  console.log("");
  console.log(`AWS Cost Report: ${accountAlias}`);
  console.log("");
  console.log(`Last Month: $${totals.totals.lastMonth.toFixed(2)}`);
  console.log(`This Month: $${totals.totals.thisMonth.toFixed(2)}`);
  console.log(`Last 7 Days: $${totals.totals.last7Days.toFixed(2)}`);
  console.log(`Yesterday: $${totals.totals.yesterday.toFixed(2)}`);
}
function printPlainText(accountAlias, totals, isSummary = false) {
  printPlainSummary(accountAlias, totals);
  if (isSummary) {
    return;
  }
  const allServices = Object.keys(totals.totalsByService.lastMonth).sort((a, b) => b.length - a.length);
  console.log("");
  console.log("Breakdown by Service:");
  console.log("");
  for (const service of allServices) {
    const lastMonth = totals.totalsByService.lastMonth[service];
    const thisMonth = totals.totalsByService.thisMonth[service];
    const last7Days = totals.totalsByService.last7Days[service];
    const yesterday = totals.totalsByService.yesterday[service];
    console.log(`${service}:`);
    console.log(
      `Last Month: $${lastMonth.toFixed(2)}, This month: $${thisMonth.toFixed(2)}, Last 7 Days: $${last7Days.toFixed(
        2
      )}, Yeseterday: $${yesterday.toFixed(2)}`
    );
  }
}

// src/index.ts
var program = new Command();
program.version(package_default.version).name(package_default.name).description(package_default.description).option("-c, --config [path]", "Path to the config file").option("-j, --json", "Get the output as JSON").option("-s, --summary", "Get only the summary without service breakdown").option("-t, --text", "Get the output as plain text (no colors / tables)").option("-v, --version", "Get the version of the CLI").option("-h, --help", "Get the help of the CLI").parse(process.argv);
var options = program.opts();
if (options.help) {
  program.help();
  process.exit(0);
}
loadConfig(options.config);
var awsConfig = getAwsCredentials();
var alias = await getAccountAlias(awsConfig);
var costs = await getTotalCosts(awsConfig);
if (options.json) {
  printJson(alias, costs, options.summary);
} else if (options.text) {
  printPlainText(alias, costs, options.summary);
} else {
  printFancy(alias, costs, options.summary);
}
//# sourceMappingURL=index.js.map